package com.emall.flash.service.shop;


import com.emall.flash.bean.constant.CfgKey;
import com.emall.flash.bean.entity.shop.*;
import com.emall.flash.bean.enumeration.shop.OrderEnum;
import com.emall.flash.bean.vo.SpringContextHolder;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.cache.CacheDao;
import com.emall.flash.dao.shop.OrderItemRepository;
import com.emall.flash.dao.shop.OrderLogRepository;
import com.emall.flash.dao.shop.OrderRepository;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.BaseService;
import com.emall.flash.service.api.express.ExpressApi;
import com.emall.flash.service.system.CfgService;
import com.emall.flash.utils.RedisCons;
import com.emall.flash.utils.StringUtil;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Service
public class OrderService extends BaseService<Order, Long, OrderRepository> {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private OrderItemRepository orderItemRepository;
    @Autowired
    private OrderLogRepository orderLogRepository;
    @Autowired
    private ExpressInfoService expressInfoService;
    @Autowired
    private GoodsService goodsService;
    @Autowired
    private GoodsSkuService goodsSkuService;
    @Autowired
    private CfgService cfgService;
    @Autowired
    private OrderRepository orderRepository;
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private CacheDao cacheDao;

    /**
     * 获取唯一订单号
     * 时间戳+随机数<br>
     * 建议生产环境使用redis获取唯一订单号
     *
     * @return
     */
    public String getOrderSn() {
        return StringUtil.getNewOrderNo();
    }

    public void save(Order order, List<OrderItem> itemList, Long idGoods) {
        boolean isGetLock = false;
        RLock lock = redissonClient.getLock(RedisCons.GOODS_INFO + idGoods);
        try {
            isGetLock = lock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                    Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
            if (isGetLock) {
                order.setOrderSn(getOrderSn());
                insert(order);
                for (OrderItem item : itemList) {
                    item.setIdOrder(order.getId());
                    //减库存
                    if(item.getIdSku()!=null) {
                        Integer skuCount = cacheDao.hget(RedisCons.GOODS_SKU_COUNT ,item.getIdSku(),Integer.class);
                        if((skuCount-item.getCount())<0){
                            throw  new RuntimeException("库存不足");
                        }
                        cacheDao.hset(RedisCons.GOODS_SKU_COUNT, item.getIdSku(),skuCount-item.getCount());
                        //更新商品的库存
                        Integer count = cacheDao.hget(RedisCons.GOODS_COUNT ,idGoods,Integer.class);
                        cacheDao.hset(RedisCons.GOODS_COUNT, idGoods,count-item.getCount());
                    }else{
                        Integer count = cacheDao.hget(RedisCons.GOODS_COUNT ,idGoods,Integer.class);
                        //更新商品库存
                        if((count-item.getCount())<0){
                            throw  new RuntimeException("库存不足");
                        }
                        cacheDao.hset(RedisCons.GOODS_COUNT, idGoods,count-item.getCount());
                    }

                }
                orderItemRepository.saveAll(itemList);
                OrderLog orderLog = new OrderLog();
                orderLog.setIdOrder(order.getId());
                orderLog.setDescript("生成订单");
                orderLogRepository.save(orderLog);

            }
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            isGetLock = false;
        } finally {
            lock.unlock();
            logger.error("释放成功");
        }
    }

    public void updateOrder(Order order) {
        update(order);
    }

    /**
     * 用户取消订单
     *
     * @param orderSn
     */
    public void cancel(String orderSn) {
        Order order = getByOrderSn(orderSn);
        order.setStatus(OrderEnum.OrderStatusEnum.CANCEL.getId());
        String descript = "用户取消订单";
        saveOrderLog(order, descript);
        updateOrder(order);

    }

    public Order getByOrderSn(String orderSn) {
        return get(SearchFilter.build("orderSn", SearchFilter.Operator.EQ, orderSn));
    }

    /**
     * 确认收货
     *
     * @param orderSn
     */
    public Order confirmReceive(String orderSn) {
        Order order = getByOrderSn(orderSn);
        order.setStatus(OrderEnum.OrderStatusEnum.FINISHED.getId());
        String descript = "客户确认收货";
        saveOrderLog(order, descript);
        updateOrder(order);
        return order;
    }

    public void startPay(Order order) {
        order.setPayStatus(OrderEnum.PayStatusEnum.PAYING.getId());
        saveOrderLog(order, "客户发起支付");
        updateOrder(order);

    }

    public void saveOrderLog(Order order, String descript) {
        OrderLog orderLog = new OrderLog();
        orderLog.setIdOrder(order.getId());
        orderLog.setDescript(descript);
        orderLogRepository.save(orderLog);
    }


    /**
     * 管理员添加备注信息
     *
     * @param order
     * @param message
     */
    public void addComment(Order order, String message) {
        order.setAdminMessage(message);
        update(order);
        OrderLog orderLog = new OrderLog();
        orderLog.setIdOrder(order.getId());
        orderLog.setDescript("管理员(" + JwtUtil.getUsername() + ")添加备注：" + message);
        orderLogRepository.save(orderLog);
    }

    /**
     * 支付成功，更新订单数据
     *
     * @param order
     * @param payType
     */
    public void paySuccess(Order order, String payType) {
        order.setPayTime(new Date());
        order.setPayStatus(OrderEnum.PayStatusEnum.UN_SEND.getId());
        order.setStatus(OrderEnum.OrderStatusEnum.UN_SEND.getId());
        order.setRealPrice(order.getTotalPrice());
        order.setPayType(payType);
        updateOrder(order);
        String descript = "用户付款成功";
        saveOrderLog(order, descript);

    }

    public void send(Order order) {
        String descript = "管理员(" + JwtUtil.getUsername() + ")已发货";
        updateOrder(order);
        saveOrderLog(order, descript);
    }


    /**
     * 查询指定订单号的快递信息
     *
     * @param orderSn
     * @return
     */
    public ExpressInfo getExpressInfo(String orderSn) {
        Order order = getByOrderSn(orderSn);
        ExpressInfo expressInfo = expressInfoService.get(SearchFilter.build("idOrder", 3));

        if ((expressInfo == null
                || expressInfo.getState() != ExpressInfo.STATE_FINISH) &&
                  order.getStatus() == OrderEnum.OrderStatusEnum.SENDED.getId()) {
            //远程查询
            String expressApiProvider = cfgService.getCfgValue(CfgKey.API_EXPRESS_INFO_QUERY_PROVIDER);
            ExpressApi expressApi = SpringContextHolder.getBean(expressApiProvider);
            ExpressInfo apiResponse = expressApi.realTimeQuery(order.getShippingSn(), order.getExpress().getCode());
            if (StringUtil.isNotEmpty(apiResponse.getInfo())) {
                expressInfo = expressInfoService.get(SearchFilter.build("idOrder", order.getId()));
                if (expressInfo == null) {
                    expressInfo = new ExpressInfo();
                    expressInfo.setIdOrder(order.getId());
                    expressInfo.setExpressCompany(order.getExpress().getName());
                    expressInfo.setShippingSn(order.getShippingSn());
                    expressInfo.setState(apiResponse.getState());
                }
                expressInfo.setInfo(apiResponse.getInfo());
                expressInfo.setState(apiResponse.getState());
                expressInfoService.update(expressInfo);
            }
        }
        return expressInfo;
    }

    public List<Map> queryByTenantId(Long tenantId) {
        return orderRepository.queryByTenantId(tenantId);
    }
}

